@@ -9,10 +9,12 @@
 
 #include <QObject>
 
+class QalculateEngine;
+
 class InputManager : public QObject
 {
     Q_OBJECT
-    Q_PROPERTY(QString expression READ expression WRITE setExpression NOTIFY expressionChanged)
+    Q_PROPERTY(QString expression READ expression NOTIFY expressionChanged)
     Q_PROPERTY(QString result READ result NOTIFY resultChanged)
     Q_PROPERTY(QString binaryResult READ binaryResult NOTIFY binaryResultChanged)
     Q_PROPERTY(QString hexResult READ hexResult NOTIFY hexResultChanged)
@@ -21,22 +23,24 @@ class InputManager : public QObject
     Q_PROPERTY(bool binaryMode READ binaryMode WRITE setBinaryMode)
     Q_PROPERTY(bool canUndo READ canUndo NOTIFY canUndoChanged)
     Q_PROPERTY(bool canRedo READ canRedo NOTIFY canRedoChanged)
+    Q_PROPERTY(int historyIndex READ historyIndex WRITE setHistoryIndex)
 
 public:
     static InputManager *inst();
     const QString &expression() const;
-    void setExpression(const QString &expression);
     const QString &result() const;
     const QString &binaryResult() const;
     const QString &hexResult() const;
     bool moveFromResult() const;
     int getCursorPosition() const;
     void setCursorPosition(int position);
-    Q_INVOKABLE int idealCursorPosition(int position) const;
+    Q_INVOKABLE int idealCursorPosition(int position, int arrow = 0) const;
     Q_INVOKABLE void append(const QString &subexpression);
     Q_INVOKABLE void backspace();
     Q_INVOKABLE void equal();
     Q_INVOKABLE void clear(bool save = true);
+    Q_INVOKABLE void setHistoryIndex(const int &index);
+    int historyIndex() const;
     Q_INVOKABLE void fromHistory(const QString &result);
     void store();
     Q_INVOKABLE void undo();
@@ -46,8 +50,9 @@ public:
     void setBinaryMode(bool active);
     bool binaryMode();
     QString formatNumbers(const QString &text);
+    void replaceWithSuperscript(QString &text);
     void addNumberSeparators(QString &number);
-    void calculate();
+    void calculate(bool exact = false, const int minExp = -1);
 
 Q_SIGNALS:
     void expressionChanged();
@@ -73,4 +78,7 @@ private:
     QString m_decimalPoint;
     std::vector<QString> m_undoStack;
     size_t m_undoPos = 0;
+    QalculateEngine *m_engine;
+    bool m_isApproximate;
+    int m_historyIndex;
 };
